@@ -1,4 +1,8 @@
 from __future__ import annotations
+
+from pathlib import Path
+from typing import IO
+
 import pytest
 
 from PIL import GribStubImagePlugin, Image
@@ -8,7 +12,7 @@ from .helper import hopper
 TEST_FILE = "Tests/images/WAlaska.wind.7days.grb"
 
 
-def test_open():
+def test_open() -> None:
     # Act
     with Image.open(TEST_FILE) as im:
         # Assert
@@ -19,7 +23,7 @@ def test_open():
         assert im.size == (1, 1)
 
 
-def test_invalid_file():
+def test_invalid_file() -> None:
     # Arrange
     invalid_file = "Tests/images/flower.jpg"
 
@@ -28,7 +32,7 @@ def test_invalid_file():
         GribStubImagePlugin.GribStubImageFile(invalid_file)
 
 
-def test_load():
+def test_load() -> None:
     # Arrange
     with Image.open(TEST_FILE) as im:
         # Act / Assert: stub cannot load without an implemented handler
@@ -36,7 +40,7 @@ def test_load():
             im.load()
 
 
-def test_save(tmp_path):
+def test_save(tmp_path: Path) -> None:
     # Arrange
     im = hopper()
     tmpfile = str(tmp_path / "temp.grib")
@@ -46,21 +50,21 @@ def test_save(tmp_path):
         im.save(tmpfile)
 
 
-def test_handler(tmp_path):
+def test_handler(tmp_path: Path) -> None:
     class TestHandler:
         opened = False
         loaded = False
         saved = False
 
-        def open(self, im):
+        def open(self, im: Image.Image) -> None:
             self.opened = True
 
-        def load(self, im):
+        def load(self, im: Image.Image) -> Image.Image:
             self.loaded = True
             im.fp.close()
             return Image.new("RGB", (1, 1))
 
-        def save(self, im, fp, filename):
+        def save(self, im: Image.Image, fp: IO[bytes], filename: str) -> None:
             self.saved = True
 
     handler = TestHandler()
