@@ -1,16 +1,18 @@
 from __future__ import annotations
+
 import base64
 import io
 import itertools
 import os
 import re
 import sys
-from collections import namedtuple
+from pathlib import Path
+from typing import Any, NamedTuple
 
 import pytest
 
 from PIL import Image, ImageFilter, ImageOps, TiffImagePlugin, TiffTags, features
-from PIL.TiffImagePlugin import SAMPLEFORMAT, STRIPOFFSETS, SUBIFD
+from PIL.TiffImagePlugin import OSUBFILETYPE, SAMPLEFORMAT, STRIPOFFSETS, SUBIFD
 
 from .helper import (
     assert_image_equal,
@@ -25,7 +27,7 @@ from .helper import (
 
 @skip_unless_feature("libtiff")
 class LibTiffTestCase:
-    def _assert_noerr(self, tmp_path, im):
+    def _assert_noerr(self, tmp_path: Path, im: TiffImagePlugin.TiffImageFile) -> None:
         """Helper tests that assert basic sanity about the g4 tiff reading"""
         # 1 bit
         assert im.mode == "1"
@@ -49,10 +51,10 @@ class LibTiffTestCase:
 
 
 class TestFileLibTiff(LibTiffTestCase):
-    def test_version(self):
+    def test_version(self) -> None:
         assert re.search(r"\d+\.\d+\.\d+$", features.version_codec("libtiff"))
 
-    def test_g4_tiff(self, tmp_path):
+    def test_g4_tiff(self, tmp_path: Path) -> None:
         """Test the ordinary file path load path"""
 
         test_file = "Tests/images/hopper_g4_500.tif"
@@ -60,12 +62,12 @@ class TestFileLibTiff(LibTiffTestCase):
             assert im.size == (500, 500)
             self._assert_noerr(tmp_path, im)
 
-    def test_g4_large(self, tmp_path):
+    def test_g4_large(self, tmp_path: Path) -> None:
         test_file = "Tests/images/pport_g4.tif"
         with Image.open(test_file) as im:
             self._assert_noerr(tmp_path, im)
 
-    def test_g4_tiff_file(self, tmp_path):
+    def test_g4_tiff_file(self, tmp_path: Path) -> None:
         """Testing the string load path"""
 
         test_file = "Tests/images/hopper_g4_500.tif"
@@ -74,7 +76,7 @@ class TestFileLibTiff(LibTiffTestCase):
                 assert im.size == (500, 500)
                 self._assert_noerr(tmp_path, im)
 
-    def test_g4_tiff_bytesio(self, tmp_path):
+    def test_g4_tiff_bytesio(self, tmp_path: Path) -> None:
         """Testing the stringio loading code path"""
         test_file = "Tests/images/hopper_g4_500.tif"
         s = io.BytesIO()
@@ -85,7 +87,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert im.size == (500, 500)
             self._assert_noerr(tmp_path, im)
 
-    def test_g4_non_disk_file_object(self, tmp_path):
+    def test_g4_non_disk_file_object(self, tmp_path: Path) -> None:
         """Testing loading from non-disk non-BytesIO file object"""
         test_file = "Tests/images/hopper_g4_500.tif"
         s = io.BytesIO()
@@ -97,18 +99,18 @@ class TestFileLibTiff(LibTiffTestCase):
             assert im.size == (500, 500)
             self._assert_noerr(tmp_path, im)
 
-    def test_g4_eq_png(self):
+    def test_g4_eq_png(self) -> None:
         """Checking that we're actually getting the data that we expect"""
         with Image.open("Tests/images/hopper_bw_500.png") as png:
             assert_image_equal_tofile(png, "Tests/images/hopper_g4_500.tif")
 
     # see https://github.com/python-pillow/Pillow/issues/279
-    def test_g4_fillorder_eq_png(self):
+    def test_g4_fillorder_eq_png(self) -> None:
         """Checking that we're actually getting the data that we expect"""
         with Image.open("Tests/images/g4-fillorder-test.tif") as g4:
             assert_image_equal_tofile(g4, "Tests/images/g4-fillorder-test.png")
 
-    def test_g4_write(self, tmp_path):
+    def test_g4_write(self, tmp_path: Path) -> None:
         """Checking to see that the saved image is the same as what we wrote"""
         test_file = "Tests/images/hopper_g4_500.tif"
         with Image.open(test_file) as orig:
@@ -127,7 +129,7 @@ class TestFileLibTiff(LibTiffTestCase):
 
                 assert orig.tobytes() != reread.tobytes()
 
-    def test_adobe_deflate_tiff(self):
+    def test_adobe_deflate_tiff(self) -> None:
         test_file = "Tests/images/tiff_adobe_deflate.tif"
         with Image.open(test_file) as im:
             assert im.mode == "RGB"
@@ -138,7 +140,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert_image_equal_tofile(im, "Tests/images/tiff_adobe_deflate.png")
 
     @pytest.mark.parametrize("legacy_api", (False, True))
-    def test_write_metadata(self, legacy_api, tmp_path):
+    def test_write_metadata(self, legacy_api: bool, tmp_path: Path) -> None:
         """Test metadata writing through libtiff"""
         f = str(tmp_path / "temp.tiff")
         with Image.open("Tests/images/hopper_g4.tif") as img:
@@ -183,7 +185,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert field in reloaded, f"{field} not in metadata"
 
     @pytest.mark.valgrind_known_error(reason="Known invalid metadata")
-    def test_additional_metadata(self, tmp_path):
+    def test_additional_metadata(self, tmp_path: Path) -> None:
         # these should not crash. Seriously dummy data, most of it doesn't make
         # any sense, so we're running up against limits where we're asking
         # libtiff to do stupid things.
@@ -240,37 +242,41 @@ class TestFileLibTiff(LibTiffTestCase):
 
         TiffImagePlugin.WRITE_LIBTIFF = False
 
-    def test_custom_metadata(self, tmp_path):
-        tc = namedtuple("test_case", "value,type,supported_by_default")
+    def test_custom_metadata(self, tmp_path: Path) -> None:
+        class Tc(NamedTuple):
+            value: Any
+            type: int
+            supported_by_default: bool
+
         custom = {
             37000 + k: v
             for k, v in enumerate(
                 [
-                    tc(4, TiffTags.SHORT, True),
-                    tc(123456789, TiffTags.LONG, True),
-                    tc(-4, TiffTags.SIGNED_BYTE, False),
-                    tc(-4, TiffTags.SIGNED_SHORT, False),
-                    tc(-123456789, TiffTags.SIGNED_LONG, False),
-                    tc(TiffImagePlugin.IFDRational(4, 7), TiffTags.RATIONAL, True),
-                    tc(4.25, TiffTags.FLOAT, True),
-                    tc(4.25, TiffTags.DOUBLE, True),
-                    tc("custom tag value", TiffTags.ASCII, True),
-                    tc(b"custom tag value", TiffTags.BYTE, True),
-                    tc((4, 5, 6), TiffTags.SHORT, True),
-                    tc((123456789, 9, 34, 234, 219387, 92432323), TiffTags.LONG, True),
-                    tc((-4, 9, 10), TiffTags.SIGNED_BYTE, False),
-                    tc((-4, 5, 6), TiffTags.SIGNED_SHORT, False),
-                    tc(
+                    Tc(4, TiffTags.SHORT, True),
+                    Tc(123456789, TiffTags.LONG, True),
+                    Tc(-4, TiffTags.SIGNED_BYTE, False),
+                    Tc(-4, TiffTags.SIGNED_SHORT, False),
+                    Tc(-123456789, TiffTags.SIGNED_LONG, False),
+                    Tc(TiffImagePlugin.IFDRational(4, 7), TiffTags.RATIONAL, True),
+                    Tc(4.25, TiffTags.FLOAT, True),
+                    Tc(4.25, TiffTags.DOUBLE, True),
+                    Tc("custom tag value", TiffTags.ASCII, True),
+                    Tc(b"custom tag value", TiffTags.BYTE, True),
+                    Tc((4, 5, 6), TiffTags.SHORT, True),
+                    Tc((123456789, 9, 34, 234, 219387, 92432323), TiffTags.LONG, True),
+                    Tc((-4, 9, 10), TiffTags.SIGNED_BYTE, False),
+                    Tc((-4, 5, 6), TiffTags.SIGNED_SHORT, False),
+                    Tc(
                         (-123456789, 9, 34, 234, 219387, -92432323),
                         TiffTags.SIGNED_LONG,
                         False,
                     ),
-                    tc((4.25, 5.25), TiffTags.FLOAT, True),
-                    tc((4.25, 5.25), TiffTags.DOUBLE, True),
+                    Tc((4.25, 5.25), TiffTags.FLOAT, True),
+                    Tc((4.25, 5.25), TiffTags.DOUBLE, True),
                     # array of TIFF_BYTE requires bytes instead of tuple for backwards
                     # compatibility
-                    tc(bytes([4]), TiffTags.BYTE, True),
-                    tc(bytes((4, 9, 10)), TiffTags.BYTE, True),
+                    Tc(bytes([4]), TiffTags.BYTE, True),
+                    Tc(bytes((4, 9, 10)), TiffTags.BYTE, True),
                 ]
             )
         }
@@ -282,7 +288,9 @@ class TestFileLibTiff(LibTiffTestCase):
         for libtiff in libtiffs:
             TiffImagePlugin.WRITE_LIBTIFF = libtiff
 
-            def check_tags(tiffinfo):
+            def check_tags(
+                tiffinfo: TiffImagePlugin.ImageFileDirectory_v2 | dict[int, str]
+            ) -> None:
                 im = hopper()
 
                 out = str(tmp_path / "temp.tif")
@@ -321,7 +329,13 @@ class TestFileLibTiff(LibTiffTestCase):
             )
         TiffImagePlugin.WRITE_LIBTIFF = False
 
-    def test_subifd(self, tmp_path):
+    def test_osubfiletype(self, tmp_path: Path) -> None:
+        outfile = str(tmp_path / "temp.tif")
+        with Image.open("Tests/images/g4_orientation_6.tif") as im:
+            im.tag_v2[OSUBFILETYPE] = 1
+            im.save(outfile)
+
+    def test_subifd(self, tmp_path: Path) -> None:
         outfile = str(tmp_path / "temp.tif")
         with Image.open("Tests/images/g4_orientation_6.tif") as im:
             im.tag_v2[SUBIFD] = 10000
@@ -329,7 +343,7 @@ class TestFileLibTiff(LibTiffTestCase):
             # Should not segfault
             im.save(outfile)
 
-    def test_xmlpacket_tag(self, tmp_path):
+    def test_xmlpacket_tag(self, tmp_path: Path) -> None:
         TiffImagePlugin.WRITE_LIBTIFF = True
 
         out = str(tmp_path / "temp.tif")
@@ -340,7 +354,7 @@ class TestFileLibTiff(LibTiffTestCase):
             if 700 in reloaded.tag_v2:
                 assert reloaded.tag_v2[700] == b"xmlpacket tag"
 
-    def test_int_dpi(self, tmp_path):
+    def test_int_dpi(self, tmp_path: Path) -> None:
         # issue #1765
         im = hopper("RGB")
         out = str(tmp_path / "temp.tif")
@@ -350,7 +364,7 @@ class TestFileLibTiff(LibTiffTestCase):
         with Image.open(out) as reloaded:
             assert reloaded.info["dpi"] == (72.0, 72.0)
 
-    def test_g3_compression(self, tmp_path):
+    def test_g3_compression(self, tmp_path: Path) -> None:
         with Image.open("Tests/images/hopper_g4_500.tif") as i:
             out = str(tmp_path / "temp.tif")
             i.save(out, compression="group3")
@@ -359,7 +373,7 @@ class TestFileLibTiff(LibTiffTestCase):
                 assert reread.info["compression"] == "group3"
                 assert_image_equal(reread, i)
 
-    def test_little_endian(self, tmp_path):
+    def test_little_endian(self, tmp_path: Path) -> None:
         with Image.open("Tests/images/16bit.deflate.tif") as im:
             assert im.getpixel((0, 0)) == 480
             assert im.mode == "I;16"
@@ -378,7 +392,7 @@ class TestFileLibTiff(LibTiffTestCase):
         # UNDONE - libtiff defaults to writing in native endian, so
         # on big endian, we'll get back mode = 'I;16B' here.
 
-    def test_big_endian(self, tmp_path):
+    def test_big_endian(self, tmp_path: Path) -> None:
         with Image.open("Tests/images/16bit.MM.deflate.tif") as im:
             assert im.getpixel((0, 0)) == 480
             assert im.mode == "I;16B"
@@ -395,7 +409,7 @@ class TestFileLibTiff(LibTiffTestCase):
                 assert reread.info["compression"] == im.info["compression"]
                 assert reread.getpixel((0, 0)) == 480
 
-    def test_g4_string_info(self, tmp_path):
+    def test_g4_string_info(self, tmp_path: Path) -> None:
         """Tests String data in info directory"""
         test_file = "Tests/images/hopper_g4_500.tif"
         with Image.open(test_file) as orig:
@@ -408,7 +422,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert "temp.tif" == reread.tag_v2[269]
             assert "temp.tif" == reread.tag[269][0]
 
-    def test_12bit_rawmode(self):
+    def test_12bit_rawmode(self) -> None:
         """Are we generating the same interpretation
         of the image as Imagemagick is?"""
         TiffImagePlugin.READ_LIBTIFF = True
@@ -423,7 +437,7 @@ class TestFileLibTiff(LibTiffTestCase):
 
             assert_image_equal_tofile(im, "Tests/images/12in16bit.tif")
 
-    def test_blur(self, tmp_path):
+    def test_blur(self, tmp_path: Path) -> None:
         # test case from irc, how to do blur on b/w image
         # and save to compressed tif.
         out = str(tmp_path / "temp.tif")
@@ -435,7 +449,7 @@ class TestFileLibTiff(LibTiffTestCase):
 
         assert_image_equal_tofile(im, out)
 
-    def test_compressions(self, tmp_path):
+    def test_compressions(self, tmp_path: Path) -> None:
         # Test various tiff compressions and assert similar image content but reduced
         # file sizes.
         im = hopper("RGB")
@@ -461,7 +475,7 @@ class TestFileLibTiff(LibTiffTestCase):
         assert size_compressed > size_jpeg
         assert size_jpeg > size_jpeg_30
 
-    def test_tiff_jpeg_compression(self, tmp_path):
+    def test_tiff_jpeg_compression(self, tmp_path: Path) -> None:
         im = hopper("RGB")
         out = str(tmp_path / "temp.tif")
         im.save(out, compression="tiff_jpeg")
@@ -469,7 +483,7 @@ class TestFileLibTiff(LibTiffTestCase):
         with Image.open(out) as reloaded:
             assert reloaded.info["compression"] == "jpeg"
 
-    def test_tiff_deflate_compression(self, tmp_path):
+    def test_tiff_deflate_compression(self, tmp_path: Path) -> None:
         im = hopper("RGB")
         out = str(tmp_path / "temp.tif")
         im.save(out, compression="tiff_deflate")
@@ -477,7 +491,7 @@ class TestFileLibTiff(LibTiffTestCase):
         with Image.open(out) as reloaded:
             assert reloaded.info["compression"] == "tiff_adobe_deflate"
 
-    def test_quality(self, tmp_path):
+    def test_quality(self, tmp_path: Path) -> None:
         im = hopper("RGB")
         out = str(tmp_path / "temp.tif")
 
@@ -492,7 +506,7 @@ class TestFileLibTiff(LibTiffTestCase):
         im.save(out, compression="jpeg", quality=0)
         im.save(out, compression="jpeg", quality=100)
 
-    def test_cmyk_save(self, tmp_path):
+    def test_cmyk_save(self, tmp_path: Path) -> None:
         im = hopper("CMYK")
         out = str(tmp_path / "temp.tif")
 
@@ -500,7 +514,7 @@ class TestFileLibTiff(LibTiffTestCase):
         assert_image_equal_tofile(im, out)
 
     @pytest.mark.parametrize("im", (hopper("P"), Image.new("P", (1, 1), "#000")))
-    def test_palette_save(self, im, tmp_path):
+    def test_palette_save(self, im: Image.Image, tmp_path: Path) -> None:
         out = str(tmp_path / "temp.tif")
 
         TiffImagePlugin.WRITE_LIBTIFF = True
@@ -512,15 +526,16 @@ class TestFileLibTiff(LibTiffTestCase):
             assert len(reloaded.tag_v2[320]) == 768
 
     @pytest.mark.parametrize("compression", ("tiff_ccitt", "group3", "group4"))
-    def test_bw_compression_w_rgb(self, compression, tmp_path):
+    def test_bw_compression_w_rgb(self, compression: str, tmp_path: Path) -> None:
         im = hopper("RGB")
         out = str(tmp_path / "temp.tif")
 
         with pytest.raises(OSError):
             im.save(out, compression=compression)
 
-    def test_fp_leak(self):
-        im = Image.open("Tests/images/hopper_g4_500.tif")
+    def test_fp_leak(self) -> None:
+        im: Image.Image | None = Image.open("Tests/images/hopper_g4_500.tif")
+        assert im is not None
         fn = im.fp.fileno()
 
         os.fstat(fn)
@@ -533,7 +548,7 @@ class TestFileLibTiff(LibTiffTestCase):
         with pytest.raises(OSError):
             os.close(fn)
 
-    def test_multipage(self):
+    def test_multipage(self) -> None:
         # issue #862
         TiffImagePlugin.READ_LIBTIFF = True
         with Image.open("Tests/images/multipage.tiff") as im:
@@ -556,7 +571,7 @@ class TestFileLibTiff(LibTiffTestCase):
 
         TiffImagePlugin.READ_LIBTIFF = False
 
-    def test_multipage_nframes(self):
+    def test_multipage_nframes(self) -> None:
         # issue #862
         TiffImagePlugin.READ_LIBTIFF = True
         with Image.open("Tests/images/multipage.tiff") as im:
@@ -569,7 +584,7 @@ class TestFileLibTiff(LibTiffTestCase):
 
         TiffImagePlugin.READ_LIBTIFF = False
 
-    def test_multipage_seek_backwards(self):
+    def test_multipage_seek_backwards(self) -> None:
         TiffImagePlugin.READ_LIBTIFF = True
         with Image.open("Tests/images/multipage.tiff") as im:
             im.seek(1)
@@ -580,14 +595,14 @@ class TestFileLibTiff(LibTiffTestCase):
 
         TiffImagePlugin.READ_LIBTIFF = False
 
-    def test__next(self):
+    def test__next(self) -> None:
         TiffImagePlugin.READ_LIBTIFF = True
         with Image.open("Tests/images/hopper.tif") as im:
             assert not im.tag.next
             im.load()
             assert not im.tag.next
 
-    def test_4bit(self):
+    def test_4bit(self) -> None:
         # Arrange
         test_file = "Tests/images/hopper_gray_4bpp.tif"
         original = hopper("L")
@@ -602,7 +617,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert im.mode == "L"
             assert_image_similar(im, original, 7.3)
 
-    def test_gray_semibyte_per_pixel(self):
+    def test_gray_semibyte_per_pixel(self) -> None:
         test_files = (
             (
                 24.8,  # epsilon
@@ -635,7 +650,7 @@ class TestFileLibTiff(LibTiffTestCase):
                     assert im2.mode == "L"
                     assert_image_equal(im, im2)
 
-    def test_save_bytesio(self):
+    def test_save_bytesio(self) -> None:
         # PR 1011
         # Test TIFF saving to io.BytesIO() object.
 
@@ -645,7 +660,7 @@ class TestFileLibTiff(LibTiffTestCase):
         # Generate test image
         pilim = hopper()
 
-        def save_bytesio(compression=None):
+        def save_bytesio(compression: str | None = None) -> None:
             buffer_io = io.BytesIO()
             pilim.save(buffer_io, format="tiff", compression=compression)
             buffer_io.seek(0)
@@ -660,7 +675,7 @@ class TestFileLibTiff(LibTiffTestCase):
         TiffImagePlugin.WRITE_LIBTIFF = False
         TiffImagePlugin.READ_LIBTIFF = False
 
-    def test_save_ycbcr(self, tmp_path):
+    def test_save_ycbcr(self, tmp_path: Path) -> None:
         im = hopper("YCbCr")
         outfile = str(tmp_path / "temp.tif")
         im.save(outfile, compression="jpeg")
@@ -669,7 +684,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert reloaded.tag_v2[530] == (1, 1)
             assert reloaded.tag_v2[532] == (0, 255, 128, 255, 128, 255)
 
-    def test_exif_ifd(self, tmp_path):
+    def test_exif_ifd(self, tmp_path: Path) -> None:
         outfile = str(tmp_path / "temp.tif")
         with Image.open("Tests/images/tiff_adobe_deflate.tif") as im:
             assert im.tag_v2[34665] == 125456
@@ -679,7 +694,7 @@ class TestFileLibTiff(LibTiffTestCase):
             if Image.core.libtiff_support_custom_tags:
                 assert reloaded.tag_v2[34665] == 125456
 
-    def test_crashing_metadata(self, tmp_path):
+    def test_crashing_metadata(self, tmp_path: Path) -> None:
         # issue 1597
         with Image.open("Tests/images/rdf.tif") as im:
             out = str(tmp_path / "temp.tif")
@@ -689,7 +704,7 @@ class TestFileLibTiff(LibTiffTestCase):
             im.save(out, format="TIFF")
         TiffImagePlugin.WRITE_LIBTIFF = False
 
-    def test_page_number_x_0(self, tmp_path):
+    def test_page_number_x_0(self, tmp_path: Path) -> None:
         # Issue 973
         # Test TIFF with tag 297 (Page Number) having value of 0 0.
         # The first number is the current page number.
@@ -703,7 +718,7 @@ class TestFileLibTiff(LibTiffTestCase):
             # Should not divide by zero
             im.save(outfile)
 
-    def test_fd_duplication(self, tmp_path):
+    def test_fd_duplication(self, tmp_path: Path) -> None:
         # https://github.com/python-pillow/Pillow/issues/1651
 
         tmpfile = str(tmp_path / "temp.tif")
@@ -712,12 +727,13 @@ class TestFileLibTiff(LibTiffTestCase):
                 f.write(src.read())
 
         im = Image.open(tmpfile)
+        assert isinstance(im, TiffImagePlugin.TiffImageFile)
         im.n_frames
         im.close()
         # Should not raise PermissionError.
         os.remove(tmpfile)
 
-    def test_read_icc(self):
+    def test_read_icc(self) -> None:
         with Image.open("Tests/images/hopper.iccprofile.tif") as img:
             icc = img.info.get("icc_profile")
             assert icc is not None
@@ -728,8 +744,8 @@ class TestFileLibTiff(LibTiffTestCase):
         TiffImagePlugin.READ_LIBTIFF = False
         assert icc == icc_libtiff
 
-    def test_write_icc(self, tmp_path):
-        def check_write(libtiff):
+    def test_write_icc(self, tmp_path: Path) -> None:
+        def check_write(libtiff: bool) -> None:
             TiffImagePlugin.WRITE_LIBTIFF = libtiff
 
             with Image.open("Tests/images/hopper.iccprofile.tif") as img:
@@ -748,7 +764,7 @@ class TestFileLibTiff(LibTiffTestCase):
         for libtiff in libtiffs:
             check_write(libtiff)
 
-    def test_multipage_compression(self):
+    def test_multipage_compression(self) -> None:
         with Image.open("Tests/images/compression.tif") as im:
             im.seek(0)
             assert im._compression == "tiff_ccitt"
@@ -764,7 +780,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert im.size == (10, 10)
             im.load()
 
-    def test_save_tiff_with_jpegtables(self, tmp_path):
+    def test_save_tiff_with_jpegtables(self, tmp_path: Path) -> None:
         # Arrange
         outfile = str(tmp_path / "temp.tif")
 
@@ -776,7 +792,7 @@ class TestFileLibTiff(LibTiffTestCase):
             # Should not raise UnicodeDecodeError or anything else
             im.save(outfile)
 
-    def test_16bit_RGB_tiff(self):
+    def test_16bit_RGB_tiff(self) -> None:
         with Image.open("Tests/images/tiff_16bit_RGB.tiff") as im:
             assert im.mode == "RGB"
             assert im.size == (100, 40)
@@ -792,7 +808,7 @@ class TestFileLibTiff(LibTiffTestCase):
 
             assert_image_equal_tofile(im, "Tests/images/tiff_16bit_RGB_target.png")
 
-    def test_16bit_RGBa_tiff(self):
+    def test_16bit_RGBa_tiff(self) -> None:
         with Image.open("Tests/images/tiff_16bit_RGBa.tiff") as im:
             assert im.mode == "RGBA"
             assert im.size == (100, 40)
@@ -804,7 +820,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert_image_equal_tofile(im, "Tests/images/tiff_16bit_RGBa_target.png")
 
     @skip_unless_feature("jpg")
-    def test_gimp_tiff(self):
+    def test_gimp_tiff(self) -> None:
         # Read TIFF JPEG images from GIMP [@PIL168]
         filename = "Tests/images/pil168.tif"
         with Image.open(filename) as im:
@@ -817,14 +833,14 @@ class TestFileLibTiff(LibTiffTestCase):
 
             assert_image_equal_tofile(im, "Tests/images/pil168.png")
 
-    def test_sampleformat(self):
+    def test_sampleformat(self) -> None:
         # https://github.com/python-pillow/Pillow/issues/1466
         with Image.open("Tests/images/copyleft.tiff") as im:
             assert im.mode == "RGB"
 
             assert_image_equal_tofile(im, "Tests/images/copyleft.png", mode="RGB")
 
-    def test_sampleformat_write(self, tmp_path):
+    def test_sampleformat_write(self, tmp_path: Path) -> None:
         im = Image.new("F", (1, 1))
         out = str(tmp_path / "temp.tif")
         TiffImagePlugin.WRITE_LIBTIFF = True
@@ -835,7 +851,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert reloaded.mode == "F"
             assert reloaded.getexif()[SAMPLEFORMAT] == 3
 
-    def test_lzma(self, capfd):
+    def test_lzma(self, capfd: pytest.CaptureFixture[str]) -> None:
         try:
             with Image.open("Tests/images/hopper_lzma.tif") as im:
                 assert im.mode == "RGB"
@@ -851,7 +867,7 @@ class TestFileLibTiff(LibTiffTestCase):
             sys.stderr.write(captured.err)
             raise
 
-    def test_webp(self, capfd):
+    def test_webp(self, capfd: pytest.CaptureFixture[str]) -> None:
         try:
             with Image.open("Tests/images/hopper_webp.tif") as im:
                 assert im.mode == "RGB"
@@ -873,7 +889,7 @@ class TestFileLibTiff(LibTiffTestCase):
             sys.stderr.write(captured.err)
             raise
 
-    def test_lzw(self):
+    def test_lzw(self) -> None:
         with Image.open("Tests/images/hopper_lzw.tif") as im:
             assert im.mode == "RGB"
             assert im.size == (128, 128)
@@ -881,12 +897,12 @@ class TestFileLibTiff(LibTiffTestCase):
             im2 = hopper()
             assert_image_similar(im, im2, 5)
 
-    def test_strip_cmyk_jpeg(self):
+    def test_strip_cmyk_jpeg(self) -> None:
         infile = "Tests/images/tiff_strip_cmyk_jpeg.tif"
         with Image.open(infile) as im:
             assert_image_similar_tofile(im, "Tests/images/pil_sample_cmyk.jpg", 0.5)
 
-    def test_strip_cmyk_16l_jpeg(self):
+    def test_strip_cmyk_16l_jpeg(self) -> None:
         infile = "Tests/images/tiff_strip_cmyk_16l_jpeg.tif"
         with Image.open(infile) as im:
             assert_image_similar_tofile(im, "Tests/images/pil_sample_cmyk.jpg", 0.5)
@@ -894,7 +910,7 @@ class TestFileLibTiff(LibTiffTestCase):
     @mark_if_feature_version(
         pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
     )
-    def test_strip_ycbcr_jpeg_2x2_sampling(self):
+    def test_strip_ycbcr_jpeg_2x2_sampling(self) -> None:
         infile = "Tests/images/tiff_strip_ycbcr_jpeg_2x2_sampling.tif"
         with Image.open(infile) as im:
             assert_image_similar_tofile(im, "Tests/images/flower.jpg", 1.2)
@@ -902,12 +918,12 @@ class TestFileLibTiff(LibTiffTestCase):
     @mark_if_feature_version(
         pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
     )
-    def test_strip_ycbcr_jpeg_1x1_sampling(self):
+    def test_strip_ycbcr_jpeg_1x1_sampling(self) -> None:
         infile = "Tests/images/tiff_strip_ycbcr_jpeg_1x1_sampling.tif"
         with Image.open(infile) as im:
             assert_image_similar_tofile(im, "Tests/images/flower2.jpg", 0.01)
 
-    def test_tiled_cmyk_jpeg(self):
+    def test_tiled_cmyk_jpeg(self) -> None:
         infile = "Tests/images/tiff_tiled_cmyk_jpeg.tif"
         with Image.open(infile) as im:
             assert_image_similar_tofile(im, "Tests/images/pil_sample_cmyk.jpg", 0.5)
@@ -915,7 +931,7 @@ class TestFileLibTiff(LibTiffTestCase):
     @mark_if_feature_version(
         pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
     )
-    def test_tiled_ycbcr_jpeg_1x1_sampling(self):
+    def test_tiled_ycbcr_jpeg_1x1_sampling(self) -> None:
         infile = "Tests/images/tiff_tiled_ycbcr_jpeg_1x1_sampling.tif"
         with Image.open(infile) as im:
             assert_image_similar_tofile(im, "Tests/images/flower2.jpg", 0.01)
@@ -923,45 +939,45 @@ class TestFileLibTiff(LibTiffTestCase):
     @mark_if_feature_version(
         pytest.mark.valgrind_known_error, "libjpeg_turbo", "2.0", reason="Known Failing"
     )
-    def test_tiled_ycbcr_jpeg_2x2_sampling(self):
+    def test_tiled_ycbcr_jpeg_2x2_sampling(self) -> None:
         infile = "Tests/images/tiff_tiled_ycbcr_jpeg_2x2_sampling.tif"
         with Image.open(infile) as im:
             assert_image_similar_tofile(im, "Tests/images/flower.jpg", 1.5)
 
-    def test_strip_planar_rgb(self):
+    def test_strip_planar_rgb(self) -> None:
         # gdal_translate -co TILED=no -co INTERLEAVE=BAND -co COMPRESS=LZW \
         # tiff_strip_raw.tif tiff_strip_planar_lzw.tiff
         infile = "Tests/images/tiff_strip_planar_lzw.tiff"
         with Image.open(infile) as im:
             assert_image_equal_tofile(im, "Tests/images/tiff_adobe_deflate.png")
 
-    def test_tiled_planar_rgb(self):
+    def test_tiled_planar_rgb(self) -> None:
         # gdal_translate -co TILED=yes -co INTERLEAVE=BAND -co COMPRESS=LZW \
         # tiff_tiled_raw.tif tiff_tiled_planar_lzw.tiff
         infile = "Tests/images/tiff_tiled_planar_lzw.tiff"
         with Image.open(infile) as im:
             assert_image_equal_tofile(im, "Tests/images/tiff_adobe_deflate.png")
 
-    def test_tiled_planar_16bit_RGB(self):
+    def test_tiled_planar_16bit_RGB(self) -> None:
         # gdal_translate -co TILED=yes -co INTERLEAVE=BAND -co COMPRESS=LZW \
         # tiff_16bit_RGB.tiff tiff_tiled_planar_16bit_RGB.tiff
         with Image.open("Tests/images/tiff_tiled_planar_16bit_RGB.tiff") as im:
             assert_image_equal_tofile(im, "Tests/images/tiff_16bit_RGB_target.png")
 
-    def test_strip_planar_16bit_RGB(self):
+    def test_strip_planar_16bit_RGB(self) -> None:
         # gdal_translate -co TILED=no -co INTERLEAVE=BAND -co COMPRESS=LZW \
         # tiff_16bit_RGB.tiff tiff_strip_planar_16bit_RGB.tiff
         with Image.open("Tests/images/tiff_strip_planar_16bit_RGB.tiff") as im:
             assert_image_equal_tofile(im, "Tests/images/tiff_16bit_RGB_target.png")
 
-    def test_tiled_planar_16bit_RGBa(self):
+    def test_tiled_planar_16bit_RGBa(self) -> None:
         # gdal_translate -co TILED=yes \
         # -co INTERLEAVE=BAND -co COMPRESS=LZW -co ALPHA=PREMULTIPLIED \
         # tiff_16bit_RGBa.tiff tiff_tiled_planar_16bit_RGBa.tiff
         with Image.open("Tests/images/tiff_tiled_planar_16bit_RGBa.tiff") as im:
             assert_image_equal_tofile(im, "Tests/images/tiff_16bit_RGBa_target.png")
 
-    def test_strip_planar_16bit_RGBa(self):
+    def test_strip_planar_16bit_RGBa(self) -> None:
         # gdal_translate -co TILED=no \
         # -co INTERLEAVE=BAND -co COMPRESS=LZW -co ALPHA=PREMULTIPLIED \
         # tiff_16bit_RGBa.tiff tiff_strip_planar_16bit_RGBa.tiff
@@ -969,7 +985,7 @@ class TestFileLibTiff(LibTiffTestCase):
             assert_image_equal_tofile(im, "Tests/images/tiff_16bit_RGBa_target.png")
 
     @pytest.mark.parametrize("compression", (None, "jpeg"))
-    def test_block_tile_tags(self, compression, tmp_path):
+    def test_block_tile_tags(self, compression: str | None, tmp_path: Path) -> None:
         im = hopper()
         out = str(tmp_path / "temp.tif")
 
@@ -985,11 +1001,11 @@ class TestFileLibTiff(LibTiffTestCase):
             for tag in tags:
                 assert tag not in reloaded.getexif()
 
-    def test_old_style_jpeg(self):
+    def test_old_style_jpeg(self) -> None:
         with Image.open("Tests/images/old-style-jpeg-compression.tif") as im:
             assert_image_equal_tofile(im, "Tests/images/old-style-jpeg-compression.png")
 
-    def test_open_missing_samplesperpixel(self):
+    def test_open_missing_samplesperpixel(self) -> None:
         with Image.open(
             "Tests/images/old-style-jpeg-compression-no-samplesperpixel.tif"
         ) as im:
@@ -1018,21 +1034,23 @@ class TestFileLibTiff(LibTiffTestCase):
             ),
         ],
     )
-    def test_wrong_bits_per_sample(self, file_name, mode, size, tile):
+    def test_wrong_bits_per_sample(
+        self, file_name: str, mode: str, size: tuple[int, int], tile
+    ) -> None:
         with Image.open("Tests/images/" + file_name) as im:
             assert im.mode == mode
             assert im.size == size
             assert im.tile == tile
             im.load()
 
-    def test_no_rows_per_strip(self):
+    def test_no_rows_per_strip(self) -> None:
         # This image does not have a RowsPerStrip TIFF tag
         infile = "Tests/images/no_rows_per_strip.tif"
         with Image.open(infile) as im:
             im.load()
         assert im.size == (950, 975)
 
-    def test_orientation(self):
+    def test_orientation(self) -> None:
         with Image.open("Tests/images/g4_orientation_1.tif") as base_im:
             for i in range(2, 9):
                 with Image.open("Tests/images/g4_orientation_" + str(i) + ".tif") as im:
@@ -1043,7 +1061,7 @@ class TestFileLibTiff(LibTiffTestCase):
 
                     assert_image_similar(base_im, im, 0.7)
 
-    def test_exif_transpose(self):
+    def test_exif_transpose(self) -> None:
         with Image.open("Tests/images/g4_orientation_1.tif") as base_im:
             for i in range(2, 9):
                 with Image.open("Tests/images/g4_orientation_" + str(i) + ".tif") as im:
@@ -1052,7 +1070,7 @@ class TestFileLibTiff(LibTiffTestCase):
                     assert_image_similar(base_im, im, 0.7)
 
     @pytest.mark.valgrind_known_error(reason="Backtrace in Python Core")
-    def test_sampleformat_not_corrupted(self):
+    def test_sampleformat_not_corrupted(self) -> None:
         # Assert that a TIFF image with SampleFormat=UINT tag is not corrupted
         # when saving to a new file.
         # Pillow 6.0 fails with "OSError: cannot identify image file".
@@ -1073,7 +1091,7 @@ class TestFileLibTiff(LibTiffTestCase):
         with Image.open(out) as im:
             im.load()
 
-    def test_realloc_overflow(self):
+    def test_realloc_overflow(self) -> None:
         TiffImagePlugin.READ_LIBTIFF = True
         with Image.open("Tests/images/tiff_overflow_rows_per_strip.tif") as im:
             with pytest.raises(OSError) as e:
@@ -1084,41 +1102,43 @@ class TestFileLibTiff(LibTiffTestCase):
         TiffImagePlugin.READ_LIBTIFF = False
 
     @pytest.mark.parametrize("compression", ("tiff_adobe_deflate", "jpeg"))
-    def test_save_multistrip(self, compression, tmp_path):
+    def test_save_multistrip(self, compression: str, tmp_path: Path) -> None:
         im = hopper("RGB").resize((256, 256))
         out = str(tmp_path / "temp.tif")
         im.save(out, compression=compression)
 
         with Image.open(out) as im:
             # Assert that there are multiple strips
+            assert isinstance(im, TiffImagePlugin.TiffImageFile)
             assert len(im.tag_v2[STRIPOFFSETS]) > 1
 
     @pytest.mark.parametrize("argument", (True, False))
-    def test_save_single_strip(self, argument, tmp_path):
+    def test_save_single_strip(self, argument: bool, tmp_path: Path) -> None:
         im = hopper("RGB").resize((256, 256))
         out = str(tmp_path / "temp.tif")
 
         if not argument:
             TiffImagePlugin.STRIP_SIZE = 2**18
         try:
-            arguments = {"compression": "tiff_adobe_deflate"}
+            arguments: dict[str, str | int] = {"compression": "tiff_adobe_deflate"}
             if argument:
                 arguments["strip_size"] = 2**18
             im.save(out, **arguments)
 
             with Image.open(out) as im:
+                assert isinstance(im, TiffImagePlugin.TiffImageFile)
                 assert len(im.tag_v2[STRIPOFFSETS]) == 1
         finally:
             TiffImagePlugin.STRIP_SIZE = 65536
 
     @pytest.mark.parametrize("compression", ("tiff_adobe_deflate", None))
-    def test_save_zero(self, compression, tmp_path):
+    def test_save_zero(self, compression: str | None, tmp_path: Path) -> None:
         im = Image.new("RGB", (0, 0))
         out = str(tmp_path / "temp.tif")
         with pytest.raises(SystemError):
             im.save(out, compression=compression)
 
-    def test_save_many_compressed(self, tmp_path):
+    def test_save_many_compressed(self, tmp_path: Path) -> None:
         im = hopper()
         out = str(tmp_path / "temp.tif")
         for _ in range(10000):
@@ -1132,7 +1152,7 @@ class TestFileLibTiff(LibTiffTestCase):
             ("Tests/images/child_ifd_jpeg.tiff", (20,)),
         ),
     )
-    def test_get_child_images(self, path, sizes):
+    def test_get_child_images(self, path: str, sizes: tuple[int, ...]) -> None:
         with Image.open(path) as im:
             ims = im.get_child_images()
 
