@@ -1,10 +1,12 @@
 from __future__ import annotations
+
 import warnings
 from io import BytesIO
+from typing import Any, cast
 
 import pytest
 
-from PIL import Image
+from PIL import Image, MpoImagePlugin
 
 from .helper import (
     assert_image_equal,
@@ -18,18 +20,15 @@ test_files = ["Tests/images/sugarshack.mpo", "Tests/images/frozenpond.mpo"]
 pytestmark = skip_unless_feature("jpg")
 
 
-def roundtrip(im, **options):
+def roundtrip(im: Image.Image, **options: Any) -> MpoImagePlugin.MpoImageFile:
     out = BytesIO()
     im.save(out, "MPO", **options)
-    test_bytes = out.tell()
     out.seek(0)
-    im = Image.open(out)
-    im.bytes = test_bytes  # for testing only
-    return im
+    return cast(MpoImagePlugin.MpoImageFile, Image.open(out))
 
 
 @pytest.mark.parametrize("test_file", test_files)
-def test_sanity(test_file):
+def test_sanity(test_file: str) -> None:
     with Image.open(test_file) as im:
         im.load()
         assert im.mode == "RGB"
@@ -38,8 +37,8 @@ def test_sanity(test_file):
 
 
 @pytest.mark.skipif(is_pypy(), reason="Requires CPython")
-def test_unclosed_file():
-    def open():
+def test_unclosed_file() -> None:
+    def open() -> None:
         im = Image.open(test_files[0])
         im.load()
 
@@ -47,14 +46,14 @@ def test_unclosed_file():
         open()
 
 
-def test_closed_file():
+def test_closed_file() -> None:
     with warnings.catch_warnings():
         im = Image.open(test_files[0])
         im.load()
         im.close()
 
 
-def test_seek_after_close():
+def test_seek_after_close() -> None:
     im = Image.open(test_files[0])
     im.close()
 
@@ -62,14 +61,14 @@ def test_seek_after_close():
         im.seek(1)
 
 
-def test_context_manager():
+def test_context_manager() -> None:
     with warnings.catch_warnings():
         with Image.open(test_files[0]) as im:
             im.load()
 
 
 @pytest.mark.parametrize("test_file", test_files)
-def test_app(test_file):
+def test_app(test_file: str) -> None:
     # Test APP/COM reader (@PIL135)
     with Image.open(test_file) as im:
         assert im.applist[0][0] == "APP1"
@@ -81,7 +80,7 @@ def test_app(test_file):
 
 
 @pytest.mark.parametrize("test_file", test_files)
-def test_exif(test_file):
+def test_exif(test_file: str) -> None:
     with Image.open(test_file) as im_original:
         im_reloaded = roundtrip(im_original, save_all=True, exif=im_original.getexif())
 
@@ -92,9 +91,9 @@ def test_exif(test_file):
         assert info[34665] == 188
 
 
-def test_frame_size():
+def test_frame_size() -> None:
     # This image has been hexedited to contain a different size
-    # in the EXIF data of the second frame
+    # in the SOF marker of the second frame
     with Image.open("Tests/images/sugarshack_frame_size.mpo") as im:
         assert im.size == (640, 480)
 
@@ -105,7 +104,7 @@ def test_frame_size():
         assert im.size == (640, 480)
 
 
-def test_ignore_frame_size():
+def test_ignore_frame_size() -> None:
     # Ignore the different size of the second frame
     # since this is not a "Large Thumbnail" image
     with Image.open("Tests/images/ignore_frame_size.mpo") as im:
@@ -119,7 +118,7 @@ def test_ignore_frame_size():
         assert im.size == (64, 64)
 
 
-def test_parallax():
+def test_parallax() -> None:
     # Nintendo
     with Image.open("Tests/images/sugarshack.mpo") as im:
         exif = im.getexif()
@@ -132,7 +131,7 @@ def test_parallax():
         assert exif.get_ifd(0x927C)[0xB211] == -3.125
 
 
-def test_reload_exif_after_seek():
+def test_reload_exif_after_seek() -> None:
     with Image.open("Tests/images/sugarshack.mpo") as im:
         exif = im.getexif()
         del exif[296]
@@ -142,14 +141,14 @@ def test_reload_exif_after_seek():
 
 
 @pytest.mark.parametrize("test_file", test_files)
-def test_mp(test_file):
+def test_mp(test_file: str) -> None:
     with Image.open(test_file) as im:
         mpinfo = im._getmp()
         assert mpinfo[45056] == b"0100"
         assert mpinfo[45057] == 2
 
 
-def test_mp_offset():
+def test_mp_offset() -> None:
     # This image has been manually hexedited to have an IFD offset of 10
     # in APP2 data, in contrast to normal 8
     with Image.open("Tests/images/sugarshack_ifd_offset.mpo") as im:
@@ -158,7 +157,7 @@ def test_mp_offset():
         assert mpinfo[45057] == 2
 
 
-def test_mp_no_data():
+def test_mp_no_data() -> None:
     # This image has been manually hexedited to have the second frame
     # beyond the end of the file
     with Image.open("Tests/images/sugarshack_no_data.mpo") as im:
@@ -167,7 +166,7 @@ def test_mp_no_data():
 
 
 @pytest.mark.parametrize("test_file", test_files)
-def test_mp_attribute(test_file):
+def test_mp_attribute(test_file: str) -> None:
     with Image.open(test_file) as im:
         mpinfo = im._getmp()
     for frame_number, mpentry in enumerate(mpinfo[0xB002]):
@@ -184,7 +183,7 @@ def test_mp_attribute(test_file):
 
 
 @pytest.mark.parametrize("test_file", test_files)
-def test_seek(test_file):
+def test_seek(test_file: str) -> None:
     with Image.open(test_file) as im:
         assert im.tell() == 0
         # prior to first image raises an error, both blatant and borderline
@@ -208,13 +207,13 @@ def test_seek(test_file):
         assert im.tell() == 0
 
 
-def test_n_frames():
+def test_n_frames() -> None:
     with Image.open("Tests/images/sugarshack.mpo") as im:
         assert im.n_frames == 2
         assert im.is_animated
 
 
-def test_eoferror():
+def test_eoferror() -> None:
     with Image.open("Tests/images/sugarshack.mpo") as im:
         n_frames = im.n_frames
 
@@ -228,7 +227,7 @@ def test_eoferror():
 
 
 @pytest.mark.parametrize("test_file", test_files)
-def test_image_grab(test_file):
+def test_image_grab(test_file: str) -> None:
     with Image.open(test_file) as im:
         assert im.tell() == 0
         im0 = im.tobytes()
@@ -243,7 +242,7 @@ def test_image_grab(test_file):
 
 
 @pytest.mark.parametrize("test_file", test_files)
-def test_save(test_file):
+def test_save(test_file: str) -> None:
     with Image.open(test_file) as im:
         assert im.tell() == 0
         jpg0 = roundtrip(im)
@@ -254,7 +253,7 @@ def test_save(test_file):
         assert_image_similar(im, jpg1, 30)
 
 
-def test_save_all():
+def test_save_all() -> None:
     for test_file in test_files:
         with Image.open(test_file) as im:
             im_reloaded = roundtrip(im, save_all=True)
