@@ -1,6 +1,8 @@
 from __future__ import annotations
+
 import io
 import struct
+from pathlib import Path
 
 import pytest
 
@@ -12,7 +14,7 @@ from .helper import assert_deep_equal, hopper
 TAG_IDS = {info.name: info.value for info in TiffTags.TAGS_V2.values()}
 
 
-def test_rt_metadata(tmp_path):
+def test_rt_metadata(tmp_path: Path) -> None:
     """Test writing arbitrary metadata into the tiff image directory
     Use case is ImageJ private tags, one numeric, one arbitrary
     data.  https://github.com/python-pillow/Pillow/issues/291
@@ -78,7 +80,7 @@ def test_rt_metadata(tmp_path):
         assert loaded.tag_v2[ImageJMetaDataByteCounts] == (8, len(bin_data) - 8)
 
 
-def test_read_metadata():
+def test_read_metadata() -> None:
     with Image.open("Tests/images/hopper_g4.tif") as img:
         assert {
             "YResolution": IFDRational(4294967295, 113653537),
@@ -119,7 +121,7 @@ def test_read_metadata():
         } == img.tag.named()
 
 
-def test_write_metadata(tmp_path):
+def test_write_metadata(tmp_path: Path) -> None:
     """Test metadata writing through the python code"""
     with Image.open("Tests/images/hopper.tif") as img:
         f = str(tmp_path / "temp.tiff")
@@ -156,7 +158,7 @@ def test_write_metadata(tmp_path):
             assert value == reloaded[tag], f"{tag} didn't roundtrip"
 
 
-def test_change_stripbytecounts_tag_type(tmp_path):
+def test_change_stripbytecounts_tag_type(tmp_path: Path) -> None:
     out = str(tmp_path / "temp.tiff")
     with Image.open("Tests/images/hopper.tif") as im:
         info = im.tag_v2
@@ -175,19 +177,21 @@ def test_change_stripbytecounts_tag_type(tmp_path):
         assert reloaded.tag_v2.tagtype[TiffImagePlugin.STRIPBYTECOUNTS] == TiffTags.LONG
 
 
-def test_no_duplicate_50741_tag():
+def test_no_duplicate_50741_tag() -> None:
     assert TAG_IDS["MakerNoteSafety"] == 50741
     assert TAG_IDS["BestQualityScale"] == 50780
 
 
-def test_iptc(tmp_path):
+def test_iptc(tmp_path: Path) -> None:
     out = str(tmp_path / "temp.tiff")
     with Image.open("Tests/images/hopper.Lab.tif") as im:
         im.save(out)
 
 
 @pytest.mark.parametrize("value, expected", ((b"test", "test"), (1, "1")))
-def test_writing_other_types_to_ascii(value, expected, tmp_path):
+def test_writing_other_types_to_ascii(
+    value: bytes | int, expected: str, tmp_path: Path
+) -> None:
     info = TiffImagePlugin.ImageFileDirectory_v2()
 
     tag = TiffTags.TAGS_V2[271]
@@ -204,7 +208,7 @@ def test_writing_other_types_to_ascii(value, expected, tmp_path):
 
 
 @pytest.mark.parametrize("value", (1, IFDRational(1)))
-def test_writing_other_types_to_bytes(value, tmp_path):
+def test_writing_other_types_to_bytes(value: int | IFDRational, tmp_path: Path) -> None:
     im = hopper()
     info = TiffImagePlugin.ImageFileDirectory_v2()
 
@@ -220,14 +224,17 @@ def test_writing_other_types_to_bytes(value, tmp_path):
         assert reloaded.tag_v2[700] == b"\x01"
 
 
-def test_writing_other_types_to_undefined(tmp_path):
+@pytest.mark.parametrize("value", (1, IFDRational(1)))
+def test_writing_other_types_to_undefined(
+    value: int | IFDRational, tmp_path: Path
+) -> None:
     im = hopper()
     info = TiffImagePlugin.ImageFileDirectory_v2()
 
     tag = TiffTags.TAGS_V2[33723]
     assert tag.type == TiffTags.UNDEFINED
 
-    info[33723] = 1
+    info[33723] = value
 
     out = str(tmp_path / "temp.tiff")
     im.save(out, tiffinfo=info)
@@ -236,7 +243,7 @@ def test_writing_other_types_to_undefined(tmp_path):
         assert reloaded.tag_v2[33723] == b"1"
 
 
-def test_undefined_zero(tmp_path):
+def test_undefined_zero(tmp_path: Path) -> None:
     # Check that the tag has not been changed since this test was created
     tag = TiffTags.TAGS_V2[45059]
     assert tag.type == TiffTags.UNDEFINED
@@ -251,7 +258,7 @@ def test_undefined_zero(tmp_path):
     assert info[45059] == original
 
 
-def test_empty_metadata():
+def test_empty_metadata() -> None:
     f = io.BytesIO(b"II*\x00\x08\x00\x00\x00")
     head = f.read(8)
     info = TiffImagePlugin.ImageFileDirectory(head)
@@ -260,7 +267,7 @@ def test_empty_metadata():
         info.load(f)
 
 
-def test_iccprofile(tmp_path):
+def test_iccprofile(tmp_path: Path) -> None:
     # https://github.com/python-pillow/Pillow/issues/1462
     out = str(tmp_path / "temp.tiff")
     with Image.open("Tests/images/hopper.iccprofile.tif") as im:
@@ -271,7 +278,7 @@ def test_iccprofile(tmp_path):
         assert im.info["icc_profile"] == reloaded.info["icc_profile"]
 
 
-def test_iccprofile_binary():
+def test_iccprofile_binary() -> None:
     # https://github.com/python-pillow/Pillow/issues/1526
     # We should be able to load this,
     # but probably won't be able to save it.
@@ -281,19 +288,19 @@ def test_iccprofile_binary():
         assert im.info["icc_profile"]
 
 
-def test_iccprofile_save_png(tmp_path):
+def test_iccprofile_save_png(tmp_path: Path) -> None:
     with Image.open("Tests/images/hopper.iccprofile.tif") as im:
         outfile = str(tmp_path / "temp.png")
         im.save(outfile)
 
 
-def test_iccprofile_binary_save_png(tmp_path):
+def test_iccprofile_binary_save_png(tmp_path: Path) -> None:
     with Image.open("Tests/images/hopper.iccprofile_binary.tif") as im:
         outfile = str(tmp_path / "temp.png")
         im.save(outfile)
 
 
-def test_exif_div_zero(tmp_path):
+def test_exif_div_zero(tmp_path: Path) -> None:
     im = hopper()
     info = TiffImagePlugin.ImageFileDirectory_v2()
     info[41988] = TiffImagePlugin.IFDRational(0, 0)
@@ -306,7 +313,7 @@ def test_exif_div_zero(tmp_path):
         assert 0 == reloaded.tag_v2[41988].denominator
 
 
-def test_ifd_unsigned_rational(tmp_path):
+def test_ifd_unsigned_rational(tmp_path: Path) -> None:
     im = hopper()
     info = TiffImagePlugin.ImageFileDirectory_v2()
 
@@ -337,7 +344,7 @@ def test_ifd_unsigned_rational(tmp_path):
         assert 1 == reloaded.tag_v2[41493].denominator
 
 
-def test_ifd_signed_rational(tmp_path):
+def test_ifd_signed_rational(tmp_path: Path) -> None:
     im = hopper()
     info = TiffImagePlugin.ImageFileDirectory_v2()
 
@@ -380,7 +387,7 @@ def test_ifd_signed_rational(tmp_path):
         assert -1 == reloaded.tag_v2[37380].denominator
 
 
-def test_ifd_signed_long(tmp_path):
+def test_ifd_signed_long(tmp_path: Path) -> None:
     im = hopper()
     info = TiffImagePlugin.ImageFileDirectory_v2()
 
@@ -393,7 +400,7 @@ def test_ifd_signed_long(tmp_path):
         assert reloaded.tag_v2[37000] == -60000
 
 
-def test_empty_values():
+def test_empty_values() -> None:
     data = io.BytesIO(
         b"II*\x00\x08\x00\x00\x00\x03\x00\x1a\x01\x05\x00\x00\x00\x00\x00"
         b"\x00\x00\x00\x00\x1b\x01\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00"
@@ -408,7 +415,7 @@ def test_empty_values():
     assert 33432 in info
 
 
-def test_photoshop_info(tmp_path):
+def test_photoshop_info(tmp_path: Path) -> None:
     with Image.open("Tests/images/issue_2278.tif") as im:
         assert len(im.tag_v2[34377]) == 70
         assert isinstance(im.tag_v2[34377], bytes)
@@ -419,7 +426,7 @@ def test_photoshop_info(tmp_path):
         assert isinstance(reloaded.tag_v2[34377], bytes)
 
 
-def test_too_many_entries():
+def test_too_many_entries() -> None:
     ifd = TiffImagePlugin.ImageFileDirectory_v2()
 
     #    277: ("SamplesPerPixel", SHORT, 1),
@@ -431,7 +438,7 @@ def test_too_many_entries():
         assert ifd[277] == 4
 
 
-def test_tag_group_data():
+def test_tag_group_data() -> None:
     base_ifd = TiffImagePlugin.ImageFileDirectory_v2()
     interop_ifd = TiffImagePlugin.ImageFileDirectory_v2(group=40965)
     for ifd in (base_ifd, interop_ifd):
@@ -445,7 +452,7 @@ def test_tag_group_data():
     assert base_ifd.tagtype[2] != interop_ifd.tagtype[256]
 
 
-def test_empty_subifd(tmp_path):
+def test_empty_subifd(tmp_path: Path) -> None:
     out = str(tmp_path / "temp.jpg")
 
     im = hopper()
