@@ -1,4 +1,5 @@
 from __future__ import annotations
+
 import sys
 from array import array
 
@@ -9,7 +10,7 @@ from PIL import Image
 from .helper import assert_image_equal, hopper
 
 
-def test_sanity():
+def test_sanity() -> None:
     im1 = hopper()
 
     data = list(im1.getdata())
@@ -28,9 +29,9 @@ def test_sanity():
     assert_image_equal(im1, im2)
 
 
-def test_long_integers():
+def test_long_integers() -> None:
     # see bug-200802-systemerror
-    def put(value):
+    def put(value: int) -> tuple[int, int, int, int]:
         im = Image.new("RGBA", (1, 1))
         im.putdata([value])
         return im.getpixel((0, 0))
@@ -45,19 +46,19 @@ def test_long_integers():
         assert put(sys.maxsize) == (255, 255, 255, 127)
 
 
-def test_pypy_performance():
+def test_pypy_performance() -> None:
     im = Image.new("L", (256, 256))
     im.putdata(list(range(256)) * 256)
 
 
-def test_mode_with_L_with_float():
+def test_mode_with_L_with_float() -> None:
     im = Image.new("L", (1, 1), 0)
     im.putdata([2.0])
     assert im.getpixel((0, 0)) == 2
 
 
 @pytest.mark.parametrize("mode", ("I", "I;16", "I;16L", "I;16B"))
-def test_mode_i(mode):
+def test_mode_i(mode: str) -> None:
     src = hopper("L")
     data = list(src.getdata())
     im = Image.new(mode, src.size, 0)
@@ -67,7 +68,7 @@ def test_mode_i(mode):
     assert list(im.getdata()) == target
 
 
-def test_mode_F():
+def test_mode_F() -> None:
     src = hopper("L")
     data = list(src.getdata())
     im = Image.new("F", src.size, 0)
@@ -78,7 +79,7 @@ def test_mode_F():
 
 
 @pytest.mark.parametrize("mode", ("BGR;15", "BGR;16", "BGR;24"))
-def test_mode_BGR(mode):
+def test_mode_BGR(mode: str) -> None:
     data = [(16, 32, 49), (32, 32, 98)]
     im = Image.new(mode, (1, 2))
     im.putdata(data)
@@ -86,7 +87,7 @@ def test_mode_BGR(mode):
     assert list(im.getdata()) == data
 
 
-def test_array_B():
+def test_array_B() -> None:
     # shouldn't segfault
     # see https://github.com/python-pillow/Pillow/issues/1008
 
@@ -97,7 +98,7 @@ def test_array_B():
     assert len(im.getdata()) == len(arr)
 
 
-def test_array_F():
+def test_array_F() -> None:
     # shouldn't segfault
     # see https://github.com/python-pillow/Pillow/issues/1008
 
@@ -108,7 +109,7 @@ def test_array_F():
     assert len(im.getdata()) == len(arr)
 
 
-def test_not_flattened():
+def test_not_flattened() -> None:
     im = Image.new("L", (1, 1))
     with pytest.raises(TypeError):
         im.putdata([[0]])
