@@ -1,5 +1,7 @@
 from __future__ import annotations
+
 from io import BytesIO
+from pathlib import Path
 
 from PIL import Image, ImageWin
 
@@ -68,7 +70,7 @@ if is_win32():
     ]
     CreateDIBSection.restype = ctypes.wintypes.HBITMAP
 
-    def serialize_dib(bi, pixels):
+    def serialize_dib(bi, pixels) -> bytearray:
         bf = BITMAPFILEHEADER()
         bf.bfType = 0x4D42
         bf.bfOffBits = ctypes.sizeof(bf) + bi.biSize
@@ -82,7 +84,7 @@ if is_win32():
         memcpy(bp + bf.bfOffBits, pixels, bi.biSizeImage)
         return bytearray(buf)
 
-    def test_pointer(tmp_path):
+    def test_pointer(tmp_path: Path) -> None:
         im = hopper()
         (width, height) = im.size
         opath = str(tmp_path / "temp.png")
