@@ -1,4 +1,7 @@
 from __future__ import annotations
+
+from pathlib import Path
+
 import pytest
 
 from PIL import ImageQt
@@ -14,7 +17,7 @@ if ImageQt.qt_is_installed:
 
 
 @pytest.mark.parametrize("mode", ("RGB", "RGBA", "L", "P", "1"))
-def test_sanity(mode, tmp_path):
+def test_sanity(mode: str, tmp_path: Path) -> None:
     src = hopper(mode)
     data = ImageQt.toqimage(src)
 
