@@ -250,6 +250,26 @@ rgb2i(UINT8 *out_, const UINT8 *in, int xsize) {
     }
 }
 
+static void
+rgb2i16l(UINT8 *out_, const UINT8 *in, int xsize) {
+    int x;
+    for (x = 0; x < xsize; x++, in += 4) {
+        UINT8 v = CLIP16(L24(in) >> 16);
+        *out_++ = v;
+        *out_++ = v >> 8;
+    }
+}
+
+static void
+rgb2i16b(UINT8 *out_, const UINT8 *in, int xsize) {
+    int x;
+    for (x = 0; x < xsize; x++, in += 4) {
+        UINT8 v = CLIP16(L24(in) >> 16);
+        *out_++ = v >> 8;
+        *out_++ = v;
+    }
+}
+
 static void
 rgb2f(UINT8 *out_, const UINT8 *in, int xsize) {
     int x;
@@ -499,26 +519,27 @@ rgba2rgb_(UINT8 *out, const UINT8 *in, int xsize) {
 }
 
 /*
- * Conversion of RGB + single transparent color to RGBA,
- * where any pixel that matches the color will have the
- * alpha channel set to 0
+ * Conversion of RGB + single transparent color either to
+ * RGBA or LA, where any pixel matching the color will have the alpha channel set to 0, or
+ * RGBa or La, where any pixel matching the color will have all channels set to 0
  */
 
 static void
-rgbT2rgba(UINT8 *out, int xsize, int r, int g, int b) {
+rgbT2a(UINT8 *out, UINT8 *in, int xsize, int r, int g, int b, int premultiplied) {
 #ifdef WORDS_BIGENDIAN
     UINT32 trns = ((r & 0xff) << 24) | ((g & 0xff) << 16) | ((b & 0xff) << 8) | 0xff;
-    UINT32 repl = trns & 0xffffff00;
+    UINT32 repl = premultiplied ? 0 : (trns & 0xffffff00);
 #else
-    UINT32 trns = (0xff << 24) | ((b & 0xff) << 16) | ((g & 0xff) << 8) | (r & 0xff);
-    UINT32 repl = trns & 0x00ffffff;
+    UINT32 trns = (0xffU << 24) | ((b & 0xff) << 16) | ((g & 0xff) << 8) | (r & 0xff);
+    UINT32 repl = premultiplied ? 0 : (trns & 0x00ffffff);
 #endif
 
     int i;
 
-    for (i = 0; i < xsize; i++, out += sizeof(trns)) {
+    UINT8 *ref = in != NULL ? in : out;
+    for (i = 0; i < xsize; i++, ref += sizeof(trns), out += sizeof(trns)) {
         UINT32 v;
-        memcpy(&v, out, sizeof(v));
+        memcpy(&v, ref, sizeof(v));
         if (v == trns) {
             memcpy(out, &repl, sizeof(repl));
         }
@@ -878,6 +899,18 @@ I16B_L(UINT8 *out, const UINT8 *in, int xsize) {
     }
 }
 
+static void
+I16_RGB(UINT8 *out, const UINT8 *in, int xsize) {
+    int x;
+    for (x = 0; x < xsize; x++, in += 2) {
+        UINT8 v = in[1] == 0 ? in[0] : 255;
+        *out++ = v;
+        *out++ = v;
+        *out++ = v;
+        *out++ = 255;
+    }
+}
+
 static struct {
     const char *from;
     const char *to;
@@ -929,12 +962,22 @@ static struct {
     {"RGB", "1", rgb2bit},
     {"RGB", "L", rgb2l},
     {"RGB", "LA", rgb2la},
+    {"RGB", "La", rgb2la},
     {"RGB", "I", rgb2i},
+    {"RGB", "I;16", rgb2i16l},
+    {"RGB", "I;16L", rgb2i16l},
+    {"RGB", "I;16B", rgb2i16b},
+#ifdef WORDS_BIGENDIAN
+    {"RGB", "I;16N", rgb2i16b},
+#else
+    {"RGB", "I;16N", rgb2i16l},
+#endif
     {"RGB", "F", rgb2f},
     {"RGB", "BGR;15", rgb2bgr15},
     {"RGB", "BGR;16", rgb2bgr16},
     {"RGB", "BGR;24", rgb2bgr24},
     {"RGB", "RGBA", rgb2rgba},
+    {"RGB", "RGBa", rgb2rgba},
     {"RGB", "RGBX", rgb2rgba},
     {"RGB", "CMYK", rgb2cmyk},
     {"RGB", "YCbCr", ImagingConvertRGB2YCbCr},
@@ -978,6 +1021,7 @@ static struct {
 
     {"I", "I;16", I_I16L},
     {"I;16", "I", I16L_I},
+    {"I;16", "RGB", I16_RGB},
     {"L", "I;16", L_I16L},
     {"I;16", "L", I16L_L},
 
@@ -1668,16 +1712,30 @@ ImagingConvertTransparent(Imaging imIn, const char *mode, int r, int g, int b) {
     ImagingSectionCookie cookie;
     ImagingShuffler convert;
     Imaging imOut = NULL;
+    int premultiplied = 0;
+    // If the transparency matches pixels in the source image, not the converted image
+    UINT8 *source;
+    int source_transparency = 0;
     int y;
 
     if (!imIn) {
         return (Imaging)ImagingError_ModeError();
     }
 
-    if (strcmp(imIn->mode, "RGB") == 0 && strcmp(mode, "RGBA") == 0) {
+    if (strcmp(imIn->mode, "RGB") == 0 && (strcmp(mode, "RGBA") == 0 || strcmp(mode, "RGBa") == 0)) {
         convert = rgb2rgba;
+        if (strcmp(mode, "RGBa") == 0) {
+            premultiplied = 1;
+        }
+    } else if (strcmp(imIn->mode, "RGB") == 0 && (strcmp(mode, "LA") == 0 || strcmp(mode, "La") == 0)) {
+        convert = rgb2la;
+        source_transparency = 1;
+        if (strcmp(mode, "La") == 0) {
+            premultiplied = 1;
+        }
     } else if ((strcmp(imIn->mode, "1") == 0 ||
                 strcmp(imIn->mode, "I") == 0 ||
+                strcmp(imIn->mode, "I;16") == 0 ||
                 strcmp(imIn->mode, "L") == 0
                ) && (
                 strcmp(mode, "RGBA") == 0 ||
@@ -1687,6 +1745,8 @@ ImagingConvertTransparent(Imaging imIn, const char *mode, int r, int g, int b) {
             convert = bit2rgb;
         } else if (strcmp(imIn->mode, "I") == 0) {
             convert = i2rgb;
+        } else if (strcmp(imIn->mode, "I;16") == 0) {
+            convert = I16_RGB;
         } else {
             convert = l2rgb;
         }
@@ -1710,7 +1770,9 @@ ImagingConvertTransparent(Imaging imIn, const char *mode, int r, int g, int b) {
     ImagingSectionEnter(&cookie);
     for (y = 0; y < imIn->ysize; y++) {
         (*convert)((UINT8 *)imOut->image[y], (UINT8 *)imIn->image[y], imIn->xsize);
-        rgbT2rgba((UINT8 *)imOut->image[y], imIn->xsize, r, g, b);
+
+        source = source_transparency ? (UINT8 *)imIn->image[y] : NULL;
+        rgbT2a((UINT8 *)imOut->image[y], source, imIn->xsize, r, g, b, premultiplied);
     }
     ImagingSectionLeave(&cookie);
 
